@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2014-2017 Phusion Holding B.V.
+ *  Copyright (c) 2014-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -37,9 +37,9 @@
 #include <ServerKit/HttpHeaderParserState.h>
 #include <DataStructures/LString.h>
 #include <DataStructures/HashedStaticString.h>
-#include <Logging.h>
-#include <Utils/StrIntUtils.h>
-#include <Utils/Hasher.h>
+#include <LoggingKit/LoggingKit.h>
+#include <StrIntTools/StrIntUtils.h>
+#include <Algorithms/Hasher.h>
 
 namespace Passenger {
 namespace ServerKit {
@@ -71,8 +71,8 @@ private:
 				if (header->key.size == 2) {
 					// Security password. Check whether it hasn't been
 					// given before and whether it is correct.
-					if (ctx->secureModePassword.empty()
-					 || psg_lstr_cmp(&header->val, ctx->secureModePassword))
+					if (ctx->config.secureModePassword.empty()
+					 || psg_lstr_cmp(&header->val, ctx->config.secureModePassword))
 					{
 						state->secureMode = true;
 						return true;
@@ -489,6 +489,9 @@ public:
 					goto default_error;
 				}
 				break;
+			case HPE_UNEXPECTED_CONTENT_LENGTH:
+				message->aux.parseError = REQUEST_CONTAINS_CONTENT_LENGTH_AND_TRANSFER_ENCODING;
+				break;
 			default:
 				default_error:
 				message->aux.parseError = HTTP_PARSER_ERRNO_BEGIN - HTTP_PARSER_ERRNO(&state->parser);
