@@ -305,22 +305,39 @@ module PhusionPassenger
 
       if options["process_title"] && !options["process_title"].empty?
         $0 = options["process_title"] + ": " + options["app_group_name"]
+      else
+        $0 = "Passenger App: " + options["app_group_name"]
       end
 
       # If we were forked from a preloader process then clear or
-      # re-establish ActiveRecord database connections. This prevents
+      # re-establish ActiveRecord/Mongoid database connections. This prevents
       # child processes from concurrently accessing the same
       # database connection handles.
-      if forked && defined?(ActiveRecord::Base)
-        if ActiveRecord::Base.respond_to?(:clear_all_connections!)
-          ActiveRecord::Base.clear_all_connections!
-        elsif ActiveRecord::Base.respond_to?(:clear_active_connections!)
-          ActiveRecord::Base.clear_active_connections!
+      if forked
+        if defined?(ActiveRecord::Base)
+          if ActiveRecord::Base.respond_to?(:connection_handler) && !ActiveRecord::Base.connection_handler.nil? && ActiveRecord::Base.connection_handler.respond_to?(:clear_all_connections!)
+            ActiveRecord::Base.connection_handler.clear_all_connections!
+          elsif ActiveRecord::Base.respond_to?(:clear_all_connections!)
+            ActiveRecord::Base.clear_all_connections!
+          elsif ActiveRecord::Base.respond_to?(:clear_active_connections!)
+            ActiveRecord::Base.clear_active_connections!
+          end
+          begin
+            ActiveRecord::Base.establish_connection
+          rescue
+            DebugLogging.debug('ActiveRecord is not configured, start it yourself')
+          end
         end
-        begin
-          ActiveRecord::Base.establish_connection
-        rescue
-          DebugLogging.debug('ActiveRecord is not configured, start it yourself')
+
+        if defined?(Mongoid::Clients)
+          Mongoid::Clients.clients.each do |name, client|
+            client.close
+            client.reconnect
+          end
+        end
+
+        if defined?(Sequel)
+          Sequel::DATABASES.each(&:disconnect)
         end
       end
 
