@@ -577,6 +577,23 @@ module PhusionPassenger
     end
     memoize :apache2_sbindir
 
+    def self.apache2_modulesdir(options = {})
+      apxs2 = options.fetch(:apxs2, self.apxs2)
+      if apxs2.nil?
+        # macOS >= 10.13 High Sierra no longer ships apxs2, so we'll use
+        # a hardcoded default.
+        if os_name_simple == 'macosx' && os_version >= '10.13' \
+          && httpd(:apxs2 => apxs2) == '/usr/sbin/httpd'
+          '/usr/libexec/apache2'
+        else
+          nil
+        end
+      else
+        `#{apxs2} -q LIBEXECDIR`.strip
+      end
+    end
+    memoize :apache2_modulesdir
+
 
     ################ Compiler and linker flags ################
 
@@ -616,9 +633,7 @@ module PhusionPassenger
           # files. On Hongli's and Camden's systems it's inside the Xcode directory,
           # while on https://github.com/phusion/passenger/issues/1986 it's inside
           # /Library/Developer/CommandLineTools.
-          xcode_prefix = `/usr/bin/xcode-select -p`.strip
-          flags << "-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apache2"
-          flags << "-I/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk/usr/include/apache2"
+          flags << "-I#{`xcrun --show-sdk-path`.strip}/usr/include/apache2"
         end
       else
         apxs2_flags = `#{apxs2} -q CFLAGS`.strip << " -I" << `#{apxs2} -q INCLUDEDIR`.strip
@@ -672,11 +687,12 @@ module PhusionPassenger
         # Running 'file' on the Apache executable usually outputs something
         # like this:
         #
-        #   /usr/sbin/httpd: Mach-O universal binary with 4 architectures
+        #   /usr/sbin/httpd: Mach-O universal binary with 5 architectures
         #   /usr/sbin/httpd (for architecture ppc7400):     Mach-O executable ppc
         #   /usr/sbin/httpd (for architecture ppc64):       Mach-O 64-bit executable ppc64
         #   /usr/sbin/httpd (for architecture i386):        Mach-O executable i386
         #   /usr/sbin/httpd (for architecture x86_64):      Mach-O 64-bit executable x86_64
+        #   /usr/sbin/httpd (for architecture arm64e):	    Mach-O 64-bit executable arm64
         #
         # But on some machines, it may output just:
         #
@@ -685,12 +701,14 @@ module PhusionPassenger
         # (http://code.google.com/p/phusion-passenger/issues/detail?id=236)
         output = `file "#{httpd}"`.strip
         if output =~ /Mach-O fat file/ && output !~ /for architecture/
-          architectures = ["i386", "ppc", "x86_64", "ppc64"]
+          architectures = ["i386", "ppc", "x86_64", "ppc64", "arm64"]
         else
           architectures = []
           output.split("\n").grep(/for architecture/).each do |line|
             line =~ /for architecture (.*?)\)/
-            architectures << $1
+            arch = $1
+            arch = "arm64" if arch.start_with?("arm64")
+            architectures << arch
           end
         end
         # The compiler may not support all architectures in the binary.
@@ -824,8 +842,8 @@ module PhusionPassenger
             # /Library/Developer/CommandLineTools.
             xcode_prefix = `/usr/bin/xcode-select -p`.strip
             ["-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apr-1 " \
-             "-I/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk/usr/include/apache2",
-              '-lapr-1']
+             "-I#{xcode_prefix}/SDKs/MacOSX.sdk/usr/include/apr-1",
+             '-lapr-1']
           else
             ['-I/usr/include/apr-1', '-lapr-1']
           end
@@ -873,8 +891,9 @@ module PhusionPassenger
             # On macOS >= 10.13 High Sierra /usr/include no longer
             # exists.
             xcode_prefix = `/usr/bin/xcode-select -p`.strip
-            ["-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apr-1",
-              '-laprutil-1']
+            ["-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apr-1 " \
+             "-I#{xcode_prefix}/SDKs/MacOSX.sdk/usr/include/apr-1",
+             '-laprutil-1']
           else
             ['-I/usr/include/apr-1', '-laprutil-1']
           end
